@@ -1,7 +1,7 @@
 // -*- Mode: Go; indent-tabs-mode: t -*-
 
 /*
- * Copyright (C) 2017 Canonical Ltd
+ * Copyright (C) 2017-2018 Canonical Ltd
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as
@@ -19,7 +19,9 @@
 
 package main
 
-//#cgo LDFLAGS: -Wl,-Bstatic -lseccomp -Wl,-Bdynamic
+//#cgo CFLAGS: -D_FILE_OFFSET_BITS=64
+//#cgo pkg-config: libseccomp
+//#cgo LDFLAGS:
 //
 //#include <asm/ioctls.h>
 //#include <ctype.h>
@@ -66,6 +68,12 @@ package main
 //#define PF_MPLS AF_MPLS
 //#endif				// AF_MPLS
 //
+// // https://github.com/sctplab/usrsctp/blob/master/usrsctplib/usrsctp.h
+//#ifndef AF_CONN
+//#define AF_CONN 123
+//#define PF_CONN AF_CONN
+//#endif				// AF_CONN
+//
 //#ifndef PR_CAP_AMBIENT
 //#define PR_CAP_AMBIENT 47
 //#define PR_CAP_AMBIENT_IS_SET    1
@@ -141,7 +149,7 @@ import (
 	"fmt"
 	"io/ioutil"
 	"os"
-	"path/filepath"
+	"regexp"
 	"strconv"
 	"strings"
 	"syscall"
@@ -242,6 +250,8 @@ var seccompResolver = map[string]uint64{
 	"PF_MPLS": C.PF_MPLS,
 	"AF_CAN":  syscall.AF_CAN,
 	"PF_CAN":  C.PF_CAN,
+	"AF_CONN": C.AF_CONN, // 123
+	"PF_CONN": C.PF_CONN,
 
 	// man 2 socket - type
 	"SOCK_STREAM":    syscall.SOCK_STREAM,
@@ -506,6 +516,18 @@ func parseLine(line string, secFilter *seccomp.ScmpFilter) error {
 		} else if strings.HasPrefix(arg, "|") {
 			cmpOp = seccomp.CompareMaskedEqual
 			value, err = readNumber(arg[1:])
+		} else if strings.HasPrefix(arg, "u:") {
+			cmpOp = seccomp.CompareEqual
+			value, err = findUid(arg[2:])
+			if err != nil {
+				return fmt.Errorf("cannot parse token %q (line %q): %v", arg, line, err)
+			}
+		} else if strings.HasPrefix(arg, "g:") {
+			cmpOp = seccomp.CompareEqual
+			value, err = findGid(arg[2:])
+			if err != nil {
+				return fmt.Errorf("cannot parse token %q (line %q): %v", arg, line, err)
+			}
 		} else {
 			cmpOp = seccomp.CompareEqual
 			value, err = readNumber(arg)
@@ -605,17 +627,21 @@ func compile(content []byte, out string) error {
 	for scanner.Scan() {
 		line := strings.TrimSpace(scanner.Text())
 
-		// FIXME: right now complain mode is the equivalent to
-		// unrestricted.  We'll want to change this once we
-		// seccomp logging is in order.
-		//
-		// special case: unrestricted means we switch to an allow-all
-		//               filter and are done
-		if line == "@unrestricted" || line == "@complain" {
+		// special case: unrestricted means we stop early, we just
+		// write this special tag and evalulate in snap-confine
+		if line == "@unrestricted" {
+			return osutil.AtomicWrite(out, bytes.NewBufferString(line+"\n"), 0644, 0)
+		}
+		// complain mode is a "allow-all" filter for now until
+		// we can land https://github.com/snapcore/snapd/pull/3998
+		if line == "@complain" {
 			secFilter, err = seccomp.NewFilter(seccomp.ActAllow)
 			if err != nil {
 				return fmt.Errorf("cannot create seccomp filter: %s", err)
 			}
+			if err := addSecondaryArches(secFilter); err != nil {
+				return err
+			}
 			break
 		}
 
@@ -633,27 +659,36 @@ func compile(content []byte, out string) error {
 	}
 
 	// write atomically
-	dir, err := os.Open(filepath.Dir(out))
-	if err != nil {
-		return err
-	}
-	defer dir.Close()
-
-	fout, err := os.Create(out + ".tmp")
+	fout, err := osutil.NewAtomicFile(out, 0644, 0, osutil.NoChown, osutil.NoChown)
 	if err != nil {
 		return err
 	}
 	defer fout.Close()
-	if err := secFilter.ExportBPF(fout); err != nil {
+
+	if err := secFilter.ExportBPF(fout.File); err != nil {
 		return err
 	}
-	if err := fout.Sync(); err != nil {
-		return err
+	return fout.Commit()
+}
+
+// Be very strict so usernames and groups specified in policy are widely
+// compatible. From NAME_REGEX in /etc/adduser.conf
+var userGroupNamePattern = regexp.MustCompile("^[a-z][-a-z0-9_]*$")
+
+// findUid returns the identifier of the given UNIX user name.
+func findUid(username string) (uint64, error) {
+	if !userGroupNamePattern.MatchString(username) {
+		return 0, fmt.Errorf("%q must be a valid username", username)
 	}
-	if err := os.Rename(out+".tmp", out); err != nil {
-		return err
+	return osutil.FindUid(username)
+}
+
+// findGid returns the identifier of the given UNIX group name.
+func findGid(group string) (uint64, error) {
+	if !userGroupNamePattern.MatchString(group) {
+		return 0, fmt.Errorf("%q must be a valid group name", group)
 	}
-	return dir.Sync()
+	return osutil.FindGid(group)
 }
 
 func showSeccompLibraryVersion() error {
