@@ -26,6 +26,7 @@ import (
 	"github.com/snapcore/snapd/interfaces/apparmor"
 	"github.com/snapcore/snapd/interfaces/builtin"
 	"github.com/snapcore/snapd/interfaces/dbus"
+	"github.com/snapcore/snapd/interfaces/seccomp"
 	"github.com/snapcore/snapd/release"
 	"github.com/snapcore/snapd/snap"
 	"github.com/snapcore/snapd/snap/snaptest"
@@ -38,19 +39,27 @@ type DbusInterfaceSuite struct {
 
 	snapInfo *snap.Info
 
-	sessionPlug          *interfaces.Plug
-	systemPlug           *interfaces.Plug
-	connectedSessionPlug *interfaces.Plug
-	connectedSystemPlug  *interfaces.Plug
+	sessionPlugInfo          *snap.PlugInfo
+	sessionPlug              *interfaces.ConnectedPlug
+	systemPlugInfo           *snap.PlugInfo
+	systemPlug               *interfaces.ConnectedPlug
+	connectedSessionPlugInfo *snap.PlugInfo
+	connectedSessionPlug     *interfaces.ConnectedPlug
+	connectedSystemPlugInfo  *snap.PlugInfo
+	connectedSystemPlug      *interfaces.ConnectedPlug
 
-	sessionSlot          *interfaces.Slot
-	systemSlot           *interfaces.Slot
-	connectedSessionSlot *interfaces.Slot
-	connectedSystemSlot  *interfaces.Slot
+	sessionSlotInfo          *snap.SlotInfo
+	sessionSlot              *interfaces.ConnectedSlot
+	systemSlotInfo           *snap.SlotInfo
+	systemSlot               *interfaces.ConnectedSlot
+	connectedSessionSlotInfo *snap.SlotInfo
+	connectedSessionSlot     *interfaces.ConnectedSlot
+	connectedSystemSlotInfo  *snap.SlotInfo
+	connectedSystemSlot      *interfaces.ConnectedSlot
 }
 
 var _ = Suite(&DbusInterfaceSuite{
-	iface: &builtin.DbusInterface{},
+	iface: builtin.MustInterface("dbus"),
 })
 
 func (s *DbusInterfaceSuite) SetUpSuite(c *C) {
@@ -109,15 +118,23 @@ apps:
 }
 
 func (s *DbusInterfaceSuite) SetUpTest(c *C) {
-	s.sessionSlot = &interfaces.Slot{SlotInfo: s.snapInfo.Slots["test-session-slot"]}
-	s.systemSlot = &interfaces.Slot{SlotInfo: s.snapInfo.Slots["test-system-slot"]}
-	s.connectedSessionSlot = &interfaces.Slot{SlotInfo: s.snapInfo.Slots["test-session-connected-slot"]}
-	s.connectedSystemSlot = &interfaces.Slot{SlotInfo: s.snapInfo.Slots["test-system-connected-slot"]}
+	s.sessionSlotInfo = s.snapInfo.Slots["test-session-slot"]
+	s.sessionSlot = interfaces.NewConnectedSlot(s.sessionSlotInfo, nil)
+	s.systemSlotInfo = s.snapInfo.Slots["test-system-slot"]
+	s.systemSlot = interfaces.NewConnectedSlot(s.systemSlotInfo, nil)
+	s.connectedSessionSlotInfo = s.snapInfo.Slots["test-session-connected-slot"]
+	s.connectedSessionSlot = interfaces.NewConnectedSlot(s.connectedSessionSlotInfo, nil)
+	s.connectedSystemSlotInfo = s.snapInfo.Slots["test-system-connected-slot"]
+	s.connectedSystemSlot = interfaces.NewConnectedSlot(s.connectedSystemSlotInfo, nil)
 
-	s.sessionPlug = &interfaces.Plug{PlugInfo: s.snapInfo.Plugs["test-session-plug"]}
-	s.systemPlug = &interfaces.Plug{PlugInfo: s.snapInfo.Plugs["test-system-plug"]}
-	s.connectedSessionPlug = &interfaces.Plug{PlugInfo: s.snapInfo.Plugs["test-session-connected-plug"]}
-	s.connectedSystemPlug = &interfaces.Plug{PlugInfo: s.snapInfo.Plugs["test-system-connected-plug"]}
+	s.sessionPlugInfo = s.snapInfo.Plugs["test-session-plug"]
+	s.sessionPlug = interfaces.NewConnectedPlug(s.sessionPlugInfo, nil)
+	s.systemPlugInfo = s.snapInfo.Plugs["test-system-plug"]
+	s.systemPlug = interfaces.NewConnectedPlug(s.systemPlugInfo, nil)
+	s.connectedSessionPlugInfo = s.snapInfo.Plugs["test-session-connected-plug"]
+	s.connectedSessionPlug = interfaces.NewConnectedPlug(s.connectedSessionPlugInfo, nil)
+	s.connectedSystemPlugInfo = s.snapInfo.Plugs["test-system-connected-plug"]
+	s.connectedSystemPlug = interfaces.NewConnectedPlug(s.connectedSystemPlugInfo, nil)
 }
 
 func (s *DbusInterfaceSuite) TestName(c *C) {
@@ -137,9 +154,8 @@ slots:
 	info, err := snap.InfoFromSnapYaml(mockSnapYaml)
 	c.Assert(err, IsNil)
 
-	slot := &interfaces.Slot{SlotInfo: info.Slots["dbus-slot"]}
-	err = s.iface.SanitizeSlot(slot)
-	c.Assert(err, IsNil)
+	slot := info.Slots["dbus-slot"]
+	c.Assert(interfaces.BeforePrepareSlot(s.iface, slot), IsNil)
 }
 
 func (s *DbusInterfaceSuite) TestValidSystemBusName(c *C) {
@@ -155,9 +171,8 @@ slots:
 	info, err := snap.InfoFromSnapYaml(mockSnapYaml)
 	c.Assert(err, IsNil)
 
-	slot := &interfaces.Slot{SlotInfo: info.Slots["dbus-slot"]}
-	err = s.iface.SanitizeSlot(slot)
-	c.Assert(err, IsNil)
+	slot := info.Slots["dbus-slot"]
+	c.Assert(interfaces.BeforePrepareSlot(s.iface, slot), IsNil)
 }
 
 func (s *DbusInterfaceSuite) TestValidFullBusName(c *C) {
@@ -173,9 +188,8 @@ slots:
 	info, err := snap.InfoFromSnapYaml(mockSnapYaml)
 	c.Assert(err, IsNil)
 
-	slot := &interfaces.Slot{SlotInfo: info.Slots["dbus-slot"]}
-	err = s.iface.SanitizeSlot(slot)
-	c.Assert(err, IsNil)
+	slot := info.Slots["dbus-slot"]
+	c.Assert(interfaces.BeforePrepareSlot(s.iface, slot), IsNil)
 }
 
 func (s *DbusInterfaceSuite) TestNonexistentBusName(c *C) {
@@ -191,10 +205,8 @@ slots:
 	info, err := snap.InfoFromSnapYaml(mockSnapYaml)
 	c.Assert(err, IsNil)
 
-	slot := &interfaces.Slot{SlotInfo: info.Slots["dbus-slot"]}
-	err = s.iface.SanitizeSlot(slot)
-	c.Assert(err, Not(IsNil))
-	c.Assert(err, ErrorMatches, "bus 'nonexistent' must be one of 'session' or 'system'")
+	slot := info.Slots["dbus-slot"]
+	c.Assert(interfaces.BeforePrepareSlot(s.iface, slot), ErrorMatches, "bus 'nonexistent' must be one of 'session' or 'system'")
 }
 
 // If this test is failing, be sure to verify the AppArmor rules for binding to
@@ -212,10 +224,8 @@ slots:
 	info, err := snap.InfoFromSnapYaml(mockSnapYaml)
 	c.Assert(err, IsNil)
 
-	slot := &interfaces.Slot{SlotInfo: info.Slots["dbus-slot"]}
-	err = s.iface.SanitizeSlot(slot)
-	c.Assert(err, Not(IsNil))
-	c.Assert(err, ErrorMatches, "DBus bus name must not end with -NUMBER")
+	slot := info.Slots["dbus-slot"]
+	c.Assert(interfaces.BeforePrepareSlot(s.iface, slot), ErrorMatches, "DBus bus name must not end with -NUMBER")
 }
 
 func (s *DbusInterfaceSuite) TestSanitizeSlotSystem(c *C) {
@@ -231,9 +241,8 @@ slots:
 	info, err := snap.InfoFromSnapYaml(mockSnapYaml)
 	c.Assert(err, IsNil)
 
-	slot := &interfaces.Slot{SlotInfo: info.Slots["dbus-slot"]}
-	err = s.iface.SanitizeSlot(slot)
-	c.Assert(err, IsNil)
+	slot := info.Slots["dbus-slot"]
+	c.Assert(interfaces.BeforePrepareSlot(s.iface, slot), IsNil)
 }
 
 func (s *DbusInterfaceSuite) TestSanitizeSlotSession(c *C) {
@@ -249,9 +258,8 @@ slots:
 	info, err := snap.InfoFromSnapYaml(mockSnapYaml)
 	c.Assert(err, IsNil)
 
-	slot := &interfaces.Slot{SlotInfo: info.Slots["dbus-slot"]}
-	err = s.iface.SanitizeSlot(slot)
-	c.Assert(err, IsNil)
+	slot := info.Slots["dbus-slot"]
+	c.Assert(interfaces.BeforePrepareSlot(s.iface, slot), IsNil)
 }
 
 func (s *DbusInterfaceSuite) TestSanitizePlugSystem(c *C) {
@@ -267,9 +275,8 @@ plugs:
 	info, err := snap.InfoFromSnapYaml(mockSnapYaml)
 	c.Assert(err, IsNil)
 
-	plug := &interfaces.Plug{PlugInfo: info.Plugs["dbus-plug"]}
-	err = s.iface.SanitizePlug(plug)
-	c.Assert(err, IsNil)
+	plug := info.Plugs["dbus-plug"]
+	c.Assert(interfaces.BeforePreparePlug(s.iface, plug), IsNil)
 }
 
 func (s *DbusInterfaceSuite) TestSanitizePlugSession(c *C) {
@@ -285,14 +292,13 @@ plugs:
 	info, err := snap.InfoFromSnapYaml(mockSnapYaml)
 	c.Assert(err, IsNil)
 
-	plug := &interfaces.Plug{PlugInfo: info.Plugs["dbus-plug"]}
-	err = s.iface.SanitizePlug(plug)
-	c.Assert(err, IsNil)
+	plug := info.Plugs["dbus-plug"]
+	c.Assert(interfaces.BeforePreparePlug(s.iface, plug), IsNil)
 }
 
 func (s *DbusInterfaceSuite) TestPermanentSlotAppArmorSession(c *C) {
 	apparmorSpec := &apparmor.Specification{}
-	err := apparmorSpec.AddPermanentSlot(s.iface, s.sessionSlot)
+	err := apparmorSpec.AddPermanentSlot(s.iface, s.sessionSlotInfo)
 	c.Assert(err, IsNil)
 	c.Assert(apparmorSpec.SecurityTags(), DeepEquals, []string{"snap.test-dbus.test-session-provider"})
 	snippet := apparmorSpec.SnippetForTag("snap.test-dbus.test-session-provider")
@@ -316,10 +322,9 @@ func (s *DbusInterfaceSuite) TestPermanentSlotAppArmorSession(c *C) {
 func (s *DbusInterfaceSuite) TestPermanentSlotAppArmorSessionNative(c *C) {
 	restore := release.MockOnClassic(false)
 	defer restore()
-	iface := &builtin.DbusInterface{}
 
 	apparmorSpec := &apparmor.Specification{}
-	err := apparmorSpec.AddPermanentSlot(iface, s.sessionSlot)
+	err := apparmorSpec.AddPermanentSlot(s.iface, s.sessionSlotInfo)
 	c.Assert(err, IsNil)
 	c.Assert(apparmorSpec.SecurityTags(), DeepEquals, []string{"snap.test-dbus.test-session-provider"})
 
@@ -330,10 +335,9 @@ func (s *DbusInterfaceSuite) TestPermanentSlotAppArmorSessionNative(c *C) {
 func (s *DbusInterfaceSuite) TestPermanentSlotAppArmorSessionClassic(c *C) {
 	restore := release.MockOnClassic(true)
 	defer restore()
-	iface := &builtin.DbusInterface{}
 
 	apparmorSpec := &apparmor.Specification{}
-	err := apparmorSpec.AddPermanentSlot(iface, s.sessionSlot)
+	err := apparmorSpec.AddPermanentSlot(s.iface, s.sessionSlotInfo)
 	c.Assert(err, IsNil)
 	c.Assert(apparmorSpec.SecurityTags(), DeepEquals, []string{"snap.test-dbus.test-session-provider"})
 
@@ -343,7 +347,7 @@ func (s *DbusInterfaceSuite) TestPermanentSlotAppArmorSessionClassic(c *C) {
 
 func (s *DbusInterfaceSuite) TestPermanentSlotAppArmorSystem(c *C) {
 	apparmorSpec := &apparmor.Specification{}
-	err := apparmorSpec.AddPermanentSlot(s.iface, s.systemSlot)
+	err := apparmorSpec.AddPermanentSlot(s.iface, s.systemSlotInfo)
 	c.Assert(err, IsNil)
 	c.Assert(apparmorSpec.SecurityTags(), DeepEquals, []string{"snap.test-dbus.test-system-provider"})
 	snippet := apparmorSpec.SnippetForTag("snap.test-dbus.test-system-provider")
@@ -366,14 +370,14 @@ func (s *DbusInterfaceSuite) TestPermanentSlotAppArmorSystem(c *C) {
 
 func (s *DbusInterfaceSuite) TestPermanentSlotDBusSession(c *C) {
 	dbusSpec := &dbus.Specification{}
-	err := dbusSpec.AddPermanentSlot(s.iface, s.sessionSlot)
+	err := dbusSpec.AddPermanentSlot(s.iface, s.sessionSlotInfo)
 	c.Assert(err, IsNil)
 	c.Assert(dbusSpec.SecurityTags(), HasLen, 0)
 }
 
 func (s *DbusInterfaceSuite) TestPermanentSlotDBusSystem(c *C) {
 	dbusSpec := &dbus.Specification{}
-	err := dbusSpec.AddPermanentSlot(s.iface, s.systemSlot)
+	err := dbusSpec.AddPermanentSlot(s.iface, s.systemSlotInfo)
 	c.Assert(err, IsNil)
 	c.Assert(dbusSpec.SecurityTags(), DeepEquals, []string{"snap.test-dbus.test-system-provider"})
 	snippet := dbusSpec.SnippetForTag("snap.test-dbus.test-system-provider")
@@ -381,11 +385,27 @@ func (s *DbusInterfaceSuite) TestPermanentSlotDBusSystem(c *C) {
 	c.Check(snippet, testutil.Contains, "<policy context=\"default\">\n    <allow send_destination=\"org.test-system-slot\"/>")
 }
 
-func (s *DbusInterfaceSuite) TestConnectedSlotAppArmorSession(c *C) {
-	iface := &builtin.DbusInterface{}
+func (s *DbusInterfaceSuite) TestPermanentSlotSecCompSystem(c *C) {
+	seccompSpec := &seccomp.Specification{}
+	err := seccompSpec.AddPermanentSlot(s.iface, s.systemSlotInfo)
+	c.Assert(err, IsNil)
+	c.Assert(seccompSpec.SecurityTags(), DeepEquals, []string{"snap.test-dbus.test-system-provider"})
+	snippet := seccompSpec.SnippetForTag("snap.test-dbus.test-system-provider")
+	c.Check(snippet, testutil.Contains, "listen\naccept\naccept4\n")
+}
 
+func (s *DbusInterfaceSuite) TestPermanentSlotSecCompSession(c *C) {
+	seccompSpec := &seccomp.Specification{}
+	err := seccompSpec.AddPermanentSlot(s.iface, s.sessionSlotInfo)
+	c.Assert(err, IsNil)
+	c.Assert(seccompSpec.SecurityTags(), DeepEquals, []string{"snap.test-dbus.test-session-provider"})
+	snippet := seccompSpec.SnippetForTag("snap.test-dbus.test-session-provider")
+	c.Check(snippet, testutil.Contains, "listen\naccept\naccept4\n")
+}
+
+func (s *DbusInterfaceSuite) TestConnectedSlotAppArmorSession(c *C) {
 	apparmorSpec := &apparmor.Specification{}
-	err := apparmorSpec.AddConnectedSlot(iface, s.connectedSessionPlug, nil, s.connectedSessionSlot, nil)
+	err := apparmorSpec.AddConnectedSlot(s.iface, s.connectedSessionPlug, s.connectedSessionSlot)
 	c.Assert(err, IsNil)
 	c.Assert(apparmorSpec.SecurityTags(), DeepEquals, []string{"snap.test-dbus.test-session-consumer", "snap.test-dbus.test-session-provider", "snap.test-dbus.test-system-consumer", "snap.test-dbus.test-system-provider"})
 	snippet := apparmorSpec.SnippetForTag("snap.test-dbus.test-session-provider")
@@ -404,10 +424,8 @@ func (s *DbusInterfaceSuite) TestConnectedSlotAppArmorSession(c *C) {
 }
 
 func (s *DbusInterfaceSuite) TestConnectedSlotAppArmorSystem(c *C) {
-	iface := &builtin.DbusInterface{}
-
 	apparmorSpec := &apparmor.Specification{}
-	err := apparmorSpec.AddConnectedSlot(iface, s.connectedSystemPlug, nil, s.connectedSystemSlot, nil)
+	err := apparmorSpec.AddConnectedSlot(s.iface, s.connectedSystemPlug, s.connectedSystemSlot)
 	c.Assert(err, IsNil)
 	c.Assert(apparmorSpec.SecurityTags(), DeepEquals, []string{"snap.test-dbus.test-session-consumer", "snap.test-dbus.test-session-provider", "snap.test-dbus.test-system-consumer", "snap.test-dbus.test-system-provider"})
 	snippet := apparmorSpec.SnippetForTag("snap.test-dbus.test-session-provider")
@@ -426,10 +444,8 @@ func (s *DbusInterfaceSuite) TestConnectedSlotAppArmorSystem(c *C) {
 }
 
 func (s *DbusInterfaceSuite) TestConnectedPlugAppArmorSession(c *C) {
-	iface := &builtin.DbusInterface{}
-
 	apparmorSpec := &apparmor.Specification{}
-	err := apparmorSpec.AddConnectedPlug(iface, s.connectedSessionPlug, nil, s.connectedSessionSlot, nil)
+	err := apparmorSpec.AddConnectedPlug(s.iface, s.connectedSessionPlug, s.connectedSessionSlot)
 	c.Assert(err, IsNil)
 	c.Assert(apparmorSpec.SecurityTags(), DeepEquals, []string{"snap.test-dbus.test-session-consumer", "snap.test-dbus.test-session-provider", "snap.test-dbus.test-system-consumer", "snap.test-dbus.test-system-provider"})
 	snippet := apparmorSpec.SnippetForTag("snap.test-dbus.test-session-consumer")
@@ -453,10 +469,8 @@ func (s *DbusInterfaceSuite) TestConnectedPlugAppArmorSession(c *C) {
 }
 
 func (s *DbusInterfaceSuite) TestConnectedPlugAppArmorSystem(c *C) {
-	iface := &builtin.DbusInterface{}
-
 	apparmorSpec := &apparmor.Specification{}
-	err := apparmorSpec.AddConnectedPlug(iface, s.connectedSystemPlug, nil, s.connectedSystemSlot, nil)
+	err := apparmorSpec.AddConnectedPlug(s.iface, s.connectedSystemPlug, s.connectedSystemSlot)
 	c.Assert(err, IsNil)
 	c.Assert(apparmorSpec.SecurityTags(), DeepEquals, []string{"snap.test-dbus.test-session-consumer", "snap.test-dbus.test-session-provider", "snap.test-dbus.test-system-consumer", "snap.test-dbus.test-system-provider"})
 	snippet := apparmorSpec.SnippetForTag("snap.test-dbus.test-session-consumer")
@@ -503,14 +517,14 @@ slots:
 `
 
 	plugInfo := snaptest.MockInfo(c, plugYaml, nil)
-	matchingPlug := &interfaces.Plug{PlugInfo: plugInfo.Plugs["this"]}
+	matchingPlug := interfaces.NewConnectedPlug(plugInfo.Plugs["this"], nil)
 
 	slotInfo := snaptest.MockInfo(c, slotYaml, nil)
-	matchingSlot := &interfaces.Slot{SlotInfo: slotInfo.Slots["this"]}
-	nonmatchingSlot := &interfaces.Slot{SlotInfo: slotInfo.Slots["that"]}
+	matchingSlot := interfaces.NewConnectedSlot(slotInfo.Slots["this"], nil)
+	nonmatchingSlot := interfaces.NewConnectedSlot(slotInfo.Slots["that"], nil)
 
 	apparmorSpec := &apparmor.Specification{}
-	err := apparmorSpec.AddConnectedPlug(s.iface, matchingPlug, nil, matchingSlot, nil)
+	err := apparmorSpec.AddConnectedPlug(s.iface, matchingPlug, matchingSlot)
 	c.Assert(err, IsNil)
 	c.Assert(apparmorSpec.SecurityTags(), DeepEquals, []string{"snap.plugger.app"})
 	snippet := apparmorSpec.SnippetForTag("snap.plugger.app")
@@ -521,7 +535,7 @@ slots:
 	c.Check(snippet, Not(testutil.Contains), "bus=system")
 
 	apparmorSpec = &apparmor.Specification{}
-	err = apparmorSpec.AddConnectedPlug(s.iface, matchingPlug, nil, nonmatchingSlot, nil)
+	err = apparmorSpec.AddConnectedPlug(s.iface, matchingPlug, nonmatchingSlot)
 	c.Assert(err, IsNil)
 	c.Assert(apparmorSpec.SecurityTags(), HasLen, 0)
 }
@@ -552,14 +566,14 @@ slots:
 `
 
 	plugInfo := snaptest.MockInfo(c, plugYaml, nil)
-	matchingPlug := &interfaces.Plug{PlugInfo: plugInfo.Plugs["that"]}
+	matchingPlug := interfaces.NewConnectedPlug(plugInfo.Plugs["that"], nil)
 
 	slotInfo := snaptest.MockInfo(c, slotYaml, nil)
-	matchingSlot := &interfaces.Slot{SlotInfo: slotInfo.Slots["that"]}
-	nonmatchingSlot := &interfaces.Slot{SlotInfo: slotInfo.Slots["this"]}
+	matchingSlot := interfaces.NewConnectedSlot(slotInfo.Slots["that"], nil)
+	nonmatchingSlot := interfaces.NewConnectedSlot(slotInfo.Slots["this"], nil)
 
 	apparmorSpec := &apparmor.Specification{}
-	err := apparmorSpec.AddConnectedPlug(s.iface, matchingPlug, nil, matchingSlot, nil)
+	err := apparmorSpec.AddConnectedPlug(s.iface, matchingPlug, matchingSlot)
 	c.Assert(err, IsNil)
 	c.Assert(apparmorSpec.SecurityTags(), DeepEquals, []string{"snap.plugger.app"})
 	snippet := apparmorSpec.SnippetForTag("snap.plugger.app")
@@ -570,7 +584,7 @@ slots:
 	c.Check(snippet, Not(testutil.Contains), "bus=session")
 
 	apparmorSpec = &apparmor.Specification{}
-	err = apparmorSpec.AddConnectedPlug(s.iface, matchingPlug, nil, nonmatchingSlot, nil)
+	err = apparmorSpec.AddConnectedPlug(s.iface, matchingPlug, nonmatchingSlot)
 	c.Assert(err, IsNil)
 	c.Assert(apparmorSpec.SecurityTags(), HasLen, 0)
 }
@@ -605,15 +619,15 @@ slots:
 `
 
 	plugInfo := snaptest.MockInfo(c, plugYaml, nil)
-	matchingPlug1 := &interfaces.Plug{PlugInfo: plugInfo.Plugs["this"]}
-	matchingPlug2 := &interfaces.Plug{PlugInfo: plugInfo.Plugs["that"]}
+	matchingPlug1 := interfaces.NewConnectedPlug(plugInfo.Plugs["this"], nil)
+	matchingPlug2 := interfaces.NewConnectedPlug(plugInfo.Plugs["that"], nil)
 
 	slotInfo := snaptest.MockInfo(c, slotYaml, nil)
-	matchingSlot1 := &interfaces.Slot{SlotInfo: slotInfo.Slots["this"]}
-	matchingSlot2 := &interfaces.Slot{SlotInfo: slotInfo.Slots["that"]}
+	matchingSlot1 := interfaces.NewConnectedSlot(slotInfo.Slots["this"], nil)
+	matchingSlot2 := interfaces.NewConnectedSlot(slotInfo.Slots["that"], nil)
 
 	apparmorSpec := &apparmor.Specification{}
-	err := apparmorSpec.AddConnectedPlug(s.iface, matchingPlug1, nil, matchingSlot1, nil)
+	err := apparmorSpec.AddConnectedPlug(s.iface, matchingPlug1, matchingSlot1)
 	c.Assert(err, IsNil)
 	c.Assert(apparmorSpec.SecurityTags(), DeepEquals, []string{"snap.plugger.app"})
 	snippet := apparmorSpec.SnippetForTag("snap.plugger.app")
@@ -621,7 +635,7 @@ slots:
 	c.Check(snippet, testutil.Contains, "bus=session")
 
 	apparmorSpec = &apparmor.Specification{}
-	err = apparmorSpec.AddConnectedPlug(s.iface, matchingPlug2, nil, matchingSlot2, nil)
+	err = apparmorSpec.AddConnectedPlug(s.iface, matchingPlug2, matchingSlot2)
 	c.Assert(err, IsNil)
 	c.Assert(apparmorSpec.SecurityTags(), DeepEquals, []string{"snap.plugger.app"})
 	snippet = apparmorSpec.SnippetForTag("snap.plugger.app")
@@ -648,13 +662,13 @@ slots:
 `
 
 	plugInfo := snaptest.MockInfo(c, plugYaml, nil)
-	plug := &interfaces.Plug{PlugInfo: plugInfo.Plugs["this"]}
+	plug := interfaces.NewConnectedPlug(plugInfo.Plugs["this"], nil)
 
 	slotInfo := snaptest.MockInfo(c, slotYaml, nil)
-	slot := &interfaces.Slot{SlotInfo: slotInfo.Slots["this"]}
+	slot := interfaces.NewConnectedSlot(slotInfo.Slots["this"], nil)
 
 	apparmorSpec := &apparmor.Specification{}
-	err := apparmorSpec.AddConnectedPlug(s.iface, plug, nil, slot, nil)
+	err := apparmorSpec.AddConnectedPlug(s.iface, plug, slot)
 	c.Assert(err, IsNil)
 	c.Assert(apparmorSpec.SecurityTags(), HasLen, 0)
 }
@@ -678,13 +692,13 @@ slots:
 `
 
 	plugInfo := snaptest.MockInfo(c, plugYaml, nil)
-	plug := &interfaces.Plug{PlugInfo: plugInfo.Plugs["this"]}
+	plug := interfaces.NewConnectedPlug(plugInfo.Plugs["this"], nil)
 
 	slotInfo := snaptest.MockInfo(c, slotYaml, nil)
-	slot := &interfaces.Slot{SlotInfo: slotInfo.Slots["this"]}
+	slot := interfaces.NewConnectedSlot(slotInfo.Slots["this"], nil)
 
 	apparmorSpec := &apparmor.Specification{}
-	err := apparmorSpec.AddConnectedPlug(s.iface, plug, nil, slot, nil)
+	err := apparmorSpec.AddConnectedPlug(s.iface, plug, slot)
 	c.Assert(err, IsNil)
 	c.Assert(apparmorSpec.SecurityTags(), HasLen, 0)
 }
