@@ -20,16 +20,18 @@
 package snap
 
 import (
+	"bytes"
 	"fmt"
 	"io/ioutil"
 	"os"
 	"path/filepath"
+	"reflect"
 	"sort"
 	"strings"
 
 	"github.com/snapcore/snapd/dirs"
+	"github.com/snapcore/snapd/osutil/sys"
 	"github.com/snapcore/snapd/strutil"
-	"github.com/snapcore/snapd/systemd"
 	"github.com/snapcore/snapd/timeout"
 )
 
@@ -50,16 +52,25 @@ type PlaceInfo interface {
 	// DataDir returns the data directory of the snap.
 	DataDir() string
 
+	// UserDataDir returns the per user data directory of the snap.
+	UserDataDir(home string) string
+
 	// CommonDataDir returns the data directory common across revisions of the snap.
 	CommonDataDir() string
 
-	// DataHomeDir returns the per user data directory of the snap.
+	// UserCommonDataDir returns the per user data directory common across revisions of the snap.
+	UserCommonDataDir(home string) string
+
+	// UserXdgRuntimeDir returns the per user XDG_RUNTIME_DIR directory
+	UserXdgRuntimeDir(userID sys.UserID) string
+
+	// DataHomeDir returns the a glob that matches all per user data directories of a snap.
 	DataHomeDir() string
 
-	// CommonDataHomeDir returns the per user data directory common across revisions of the snap.
+	// CommonDataHomeDir returns a glob that matches all per user data directories common across revisions of the snap.
 	CommonDataHomeDir() string
 
-	// XdgRuntimeDirs returns the XDG_RUNTIME_DIR directories for all users of the snap.
+	// XdgRuntimeDirs returns a glob that matches all XDG_RUNTIME_DIR directories for all users of the snap.
 	XdgRuntimeDirs() string
 }
 
@@ -122,9 +133,11 @@ type SideInfo struct {
 	Revision          Revision `yaml:"revision" json:"revision"`
 	Channel           string   `yaml:"channel,omitempty" json:"channel,omitempty"`
 	Contact           string   `yaml:"contact,omitempty" json:"contact,omitempty"`
+	EditedTitle       string   `yaml:"title,omitempty" json:"title,omitempty"`
 	EditedSummary     string   `yaml:"summary,omitempty" json:"summary,omitempty"`
 	EditedDescription string   `yaml:"description,omitempty" json:"description,omitempty"`
 	Private           bool     `yaml:"private,omitempty" json:"private,omitempty"`
+	Paid              bool     `yaml:"paid,omitempty" json:"paid,omitempty"`
 }
 
 // Info provides information about snaps.
@@ -135,6 +148,7 @@ type Info struct {
 	Architectures []string
 	Assumes       []string
 
+	OriginalTitle       string
 	OriginalSummary     string
 	OriginalDescription string
 
@@ -142,7 +156,9 @@ type Info struct {
 
 	LicenseAgreement string
 	LicenseVersion   string
-	Epoch            string
+	License          string
+	Epoch            Epoch
+	Base             string
 	Confinement      ConfinementType
 	Apps             map[string]*AppInfo
 	LegacyAliases    map[string]*AppInfo // FIXME: eventually drop this
@@ -150,6 +166,9 @@ type Info struct {
 	Plugs            map[string]*PlugInfo
 	Slots            map[string]*SlotInfo
 
+	// Plugs or slots with issues (they are not included in Plugs or Slots)
+	BadInterfaces map[string]string // slot or plug => message
+
 	// The information in all the remaining fields is not sourced from the snap blob itself.
 	SideInfo
 
@@ -173,6 +192,21 @@ type Info struct {
 
 	// The ordered list of tracks that contain channels
 	Tracks []string
+
+	Layout map[string]*Layout
+}
+
+// Layout describes a single element of the layout section.
+type Layout struct {
+	Snap *Info
+
+	Path    string      `json:"path"`
+	Bind    string      `json:"bind,omitempty"`
+	Type    string      `json:"type,omitempty"`
+	User    string      `json:"user,omitempty"`
+	Group   string      `json:"group,omitempty"`
+	Mode    os.FileMode `json:"mode,omitempty"`
+	Symlink string      `json:"symlink,omitempty"`
 }
 
 // ChannelSnapInfo is the minimum information that can be used to clearly
@@ -182,7 +216,7 @@ type ChannelSnapInfo struct {
 	Confinement ConfinementType `json:"confinement"`
 	Version     string          `json:"version"`
 	Channel     string          `json:"channel"`
-	Epoch       string          `json:"epoch"`
+	Epoch       Epoch           `json:"epoch"`
 	Size        int64           `json:"size"`
 }
 
@@ -194,6 +228,14 @@ func (s *Info) Name() string {
 	return s.SuggestedName
 }
 
+// Title returns the blessed title for the snap.
+func (s *Info) Title() string {
+	if s.EditedTitle != "" {
+		return s.EditedTitle
+	}
+	return s.OriginalTitle
+}
+
 // Summary returns the blessed summary for the snap.
 func (s *Info) Summary() string {
 	if s.EditedSummary != "" {
@@ -235,11 +277,6 @@ func (s *Info) UserDataDir(home string) string {
 	return filepath.Join(home, "snap", s.Name(), s.Revision.String())
 }
 
-// HomeDirBase returns the user-specific home directory base of the snap.
-func (s *Info) HomeDirBase(home string) string {
-	return filepath.Join(home, "snap", s.Name())
-}
-
 // UserCommonDataDir returns the user-specific data directory common across revision of the snap.
 func (s *Info) UserCommonDataDir(home string) string {
 	return filepath.Join(home, "snap", s.Name(), "common")
@@ -261,7 +298,7 @@ func (s *Info) CommonDataHomeDir() string {
 }
 
 // UserXdgRuntimeDir returns the XDG_RUNTIME_DIR directory of the snap for a particular user.
-func (s *Info) UserXdgRuntimeDir(euid int) string {
+func (s *Info) UserXdgRuntimeDir(euid sys.UserID) string {
 	return filepath.Join("/run/user", fmt.Sprintf("%d/snap.%s", euid, s.Name()))
 }
 
@@ -280,6 +317,64 @@ func (s *Info) NeedsClassic() bool {
 	return s.Confinement == ClassicConfinement
 }
 
+// Services returns a list of the apps that have "daemon" set.
+func (s *Info) Services() []*AppInfo {
+	svcs := make([]*AppInfo, 0, len(s.Apps))
+	for _, app := range s.Apps {
+		if !app.IsService() {
+			continue
+		}
+		svcs = append(svcs, app)
+	}
+
+	return svcs
+}
+
+// ExpandSnapVariables resolves $SNAP, $SNAP_DATA and $SNAP_COMMON.
+func (s *Info) ExpandSnapVariables(path string) string {
+	return os.Expand(path, func(v string) string {
+		switch v {
+		case "SNAP":
+			// NOTE: We use dirs.CoreSnapMountDir here as the path used will be always
+			// inside the mount namespace snap-confine creates and there we will
+			// always have a /snap directory available regardless if the system
+			// we're running on supports this or not.
+			return filepath.Join(dirs.CoreSnapMountDir, s.Name(), s.Revision.String())
+		case "SNAP_DATA":
+			return s.DataDir()
+		case "SNAP_COMMON":
+			return s.CommonDataDir()
+		}
+		return ""
+	})
+}
+
+func BadInterfacesSummary(snapInfo *Info) string {
+	inverted := make(map[string][]string)
+	for name, reason := range snapInfo.BadInterfaces {
+		inverted[reason] = append(inverted[reason], name)
+	}
+	var buf bytes.Buffer
+	fmt.Fprintf(&buf, "snap %q has bad plugs or slots: ", snapInfo.Name())
+	reasons := make([]string, 0, len(inverted))
+	for reason := range inverted {
+		reasons = append(reasons, reason)
+	}
+	sort.Strings(reasons)
+	for _, reason := range reasons {
+		names := inverted[reason]
+		sort.Strings(names)
+		for i, name := range names {
+			if i > 0 {
+				buf.WriteString(", ")
+			}
+			buf.WriteString(name)
+		}
+		fmt.Fprintf(&buf, " (%s); ", reason)
+	}
+	return strings.TrimSuffix(buf.String(), "; ")
+}
+
 // DownloadInfo contains the information to download a snap.
 // It can be marshalled.
 type DownloadInfo struct {
@@ -323,6 +418,27 @@ type PlugInfo struct {
 	Hooks     map[string]*HookInfo
 }
 
+func getAttribute(snapName string, ifaceName string, attrs map[string]interface{}, key string, val interface{}) error {
+	if v, ok := attrs[key]; ok {
+		rt := reflect.TypeOf(val)
+		if rt.Kind() != reflect.Ptr || val == nil {
+			return fmt.Errorf("internal error: cannot get %q attribute of interface %q with non-pointer value", key, ifaceName)
+		}
+
+		if reflect.TypeOf(v) != rt.Elem() {
+			return fmt.Errorf("snap %q has interface %q with invalid value type for %q attribute", snapName, ifaceName, key)
+		}
+		rv := reflect.ValueOf(val)
+		rv.Elem().Set(reflect.ValueOf(v))
+		return nil
+	}
+	return fmt.Errorf("snap %q does not have attribute %q for interface %q", snapName, key, ifaceName)
+}
+
+func (plug *PlugInfo) Attr(key string, val interface{}) error {
+	return getAttribute(plug.Snap.Name(), plug.Interface, plug.Attrs, key, val)
+}
+
 // SecurityTags returns security tags associated with a given plug.
 func (plug *PlugInfo) SecurityTags() []string {
 	tags := make([]string, 0, len(plug.Apps)+len(plug.Hooks))
@@ -336,17 +452,33 @@ func (plug *PlugInfo) SecurityTags() []string {
 	return tags
 }
 
+// String returns the representation of the plug as snap:plug string.
+func (plug *PlugInfo) String() string {
+	return fmt.Sprintf("%s:%s", plug.Snap.Name(), plug.Name)
+}
+
+func (slot *SlotInfo) Attr(key string, val interface{}) error {
+	return getAttribute(slot.Snap.Name(), slot.Interface, slot.Attrs, key, val)
+}
+
 // SecurityTags returns security tags associated with a given slot.
 func (slot *SlotInfo) SecurityTags() []string {
 	tags := make([]string, 0, len(slot.Apps))
 	for _, app := range slot.Apps {
 		tags = append(tags, app.SecurityTag())
 	}
-	// NOTE: hooks cannot have slots
+	for _, hook := range slot.Hooks {
+		tags = append(tags, hook.SecurityTag())
+	}
 	sort.Strings(tags)
 	return tags
 }
 
+// String returns the representation of the slot as snap:slot string.
+func (slot *SlotInfo) String() string {
+	return fmt.Sprintf("%s:%s", slot.Snap.Name(), slot.Name)
+}
+
 // SlotInfo provides information about a slot.
 type SlotInfo struct {
 	Snap *Info
@@ -356,6 +488,16 @@ type SlotInfo struct {
 	Attrs     map[string]interface{}
 	Label     string
 	Apps      map[string]*AppInfo
+	Hooks     map[string]*HookInfo
+}
+
+// SocketInfo provides information on application sockets.
+type SocketInfo struct {
+	App *AppInfo
+
+	Name         string
+	ListenStream string
+	SocketMode   os.FileMode
 }
 
 // AppInfo provides information about a app.
@@ -371,7 +513,7 @@ type AppInfo struct {
 	StopCommand     string
 	ReloadCommand   string
 	PostStopCommand string
-	RestartCond     systemd.RestartCondition
+	RestartCond     RestartCondition
 	Completer       string
 
 	// TODO: this should go away once we have more plumbing and can change
@@ -379,10 +521,16 @@ type AppInfo struct {
 	// https://github.com/snapcore/snapd/pull/794#discussion_r58688496
 	BusName string
 
-	Plugs map[string]*PlugInfo
-	Slots map[string]*SlotInfo
+	Plugs   map[string]*PlugInfo
+	Slots   map[string]*SlotInfo
+	Sockets map[string]*SocketInfo
 
 	Environment strutil.OrderedMap
+
+	// list of other service names that this service will start after or
+	// before
+	After  []string
+	Before []string
 }
 
 // ScreenshotInfo provides information about a screenshot.
@@ -398,6 +546,12 @@ type HookInfo struct {
 
 	Name  string
 	Plugs map[string]*PlugInfo
+	Slots map[string]*SlotInfo
+}
+
+// File returns the path to the file
+func (socket *SocketInfo) File() string {
+	return filepath.Join(dirs.SnapServicesDir, socket.App.SecurityTag()+"."+socket.Name+".socket")
 }
 
 // SecurityTag returns application-specific security tag.
@@ -414,14 +568,12 @@ func (app *AppInfo) DesktopFile() string {
 
 // WrapperPath returns the path to wrapper invoking the app binary.
 func (app *AppInfo) WrapperPath() string {
-	var binName string
-	if app.Name == app.Snap.Name() {
-		binName = filepath.Base(app.Name)
-	} else {
-		binName = fmt.Sprintf("%s.%s", app.Snap.Name(), filepath.Base(app.Name))
-	}
+	return filepath.Join(dirs.SnapBinariesDir, JoinSnapApp(app.Snap.Name(), app.Name))
+}
 
-	return filepath.Join(dirs.SnapBinariesDir, binName)
+// CompleterPath returns the path to the completer snippet for the app binary.
+func (app *AppInfo) CompleterPath() string {
+	return filepath.Join(dirs.CompletersDir, JoinSnapApp(app.Snap.Name(), app.Name))
 }
 
 func (app *AppInfo) launcherCommand(command string) string {
@@ -431,7 +583,7 @@ func (app *AppInfo) launcherCommand(command string) string {
 	if app.Name == app.Snap.Name() {
 		return fmt.Sprintf("/usr/bin/snap run%s %s", command, app.Name)
 	}
-	return fmt.Sprintf("/usr/bin/snap run%s %s.%s", command, app.Snap.Name(), filepath.Base(app.Name))
+	return fmt.Sprintf("/usr/bin/snap run%s %s.%s", command, app.Snap.Name(), app.Name)
 }
 
 // LauncherCommand returns the launcher command line to use when invoking the app binary.
@@ -454,14 +606,14 @@ func (app *AppInfo) LauncherPostStopCommand() string {
 	return app.launcherCommand("--command=post-stop")
 }
 
-// ServiceFile returns the systemd service file path for the daemon app.
-func (app *AppInfo) ServiceFile() string {
-	return filepath.Join(dirs.SnapServicesDir, app.SecurityTag()+".service")
+// ServiceName returns the systemd service name for the daemon app.
+func (app *AppInfo) ServiceName() string {
+	return app.SecurityTag() + ".service"
 }
 
-// ServiceSocketFile returns the systemd socket file path for the daemon app.
-func (app *AppInfo) ServiceSocketFile() string {
-	return filepath.Join(dirs.SnapServicesDir, app.SecurityTag()+".socket")
+// ServiceFile returns the systemd service file path for the daemon app.
+func (app *AppInfo) ServiceFile() string {
+	return filepath.Join(dirs.SnapServicesDir, app.ServiceName())
 }
 
 // Env returns the app specific environment overrides
@@ -477,6 +629,11 @@ func (app *AppInfo) Env() []string {
 	return env
 }
 
+// IsService returns whether app represents a daemon/service.
+func (app *AppInfo) IsService() bool {
+	return app.Daemon != ""
+}
+
 // SecurityTag returns the hook-specific security tag.
 //
 // Security tags are used by various security subsystems as "profile names" and
@@ -517,6 +674,16 @@ func (e NotFoundError) Error() string {
 	return fmt.Sprintf("cannot find installed snap %q at revision %s", e.Snap, e.Revision)
 }
 
+func MockSanitizePlugsSlots(f func(snapInfo *Info)) (restore func()) {
+	old := SanitizePlugsSlots
+	SanitizePlugsSlots = f
+	return func() { SanitizePlugsSlots = old }
+}
+
+var SanitizePlugsSlots = func(snapInfo *Info) {
+	panic("SanitizePlugsSlots function not set")
+}
+
 // ReadInfo reads the snap information for the installed snap with the given name and given side-info.
 func ReadInfo(name string, si *SideInfo) (*Info, error) {
 	snapYamlFn := filepath.Join(MountDir(name, si.Revision), "meta", "snap.yaml")
@@ -544,6 +711,8 @@ func ReadInfo(name string, si *SideInfo) (*Info, error) {
 		return nil, err
 	}
 
+	SanitizePlugsSlots(info)
+
 	return info, nil
 }
 
